@@ -2,7 +2,7 @@
                           cmdlineoptions.cpp  -  description
                              -------------------
     begin                : Sun Oct 13 2007
-    copyright            : (C) 2007-2011 by Andre Simon
+    copyright            : (C) 2007-2017 by Andre Simon
     email                : andre.simon1@gmx.de
  ***************************************************************************/
 
@@ -26,13 +26,52 @@ along with ANSIFilter.  If not, see <http://www.gnu.org/licenses/>.
 #include <cstring>
 #include <string>
 #include <vector>
+#include <list>
 
 #include "arg_parser.h"
 #include "cmdlineoptions.h"
 #include "platform_fs.h"
 #include "stringtools.h"
 
-using namespace std;
+using namespace std;  
+    
+const Arg_parser::Option options[] = {
+    { 'a', "anchors",   Arg_parser::no },
+    { 'd', "doc-title", Arg_parser::yes },
+    { 'e', "encoding",  Arg_parser::yes },
+    { 'f', "fragment",  Arg_parser::no  },
+    { 'F', "font",      Arg_parser::yes },
+    { 'h', "help",      Arg_parser::no  },
+    { 'H', "html",      Arg_parser::no  },
+    { 'M', "pango",      Arg_parser::no  },
+    { 'i', "input",     Arg_parser::yes },
+    { 'l', "line-numbers", Arg_parser::no },
+    { 'L', "latex",     Arg_parser::no  },
+    { 'P', "tex",       Arg_parser::no  },
+    { 'B', "bbcode",    Arg_parser::no  },
+    { 'o', "output",    Arg_parser::yes },
+    { 'O', "outdir",    Arg_parser::yes },
+    { 'p', "plain",     Arg_parser::no  },
+    { 'r', "style-ref", Arg_parser::yes },
+    { 'R', "rtf",       Arg_parser::no  },
+    { 's', "font-size", Arg_parser::yes },
+    { 't', "tail",      Arg_parser::no  },
+    { 'T', "text",      Arg_parser::no  },
+    { 'w', "wrap",      Arg_parser::yes },
+    { 'v', "version",   Arg_parser::no  },
+    { 'V', "version",   Arg_parser::no  },
+    { 'W', "wrap-no-numbers", Arg_parser::no  },
+    { 'X', "art-cp437", Arg_parser::no  },
+    { 'U', "art-bin",   Arg_parser::no  },
+    { 'D', "art-tundra", Arg_parser::no  },
+    { 'Y', "art-width",  Arg_parser::yes  },
+    { 'Z', "art-height", Arg_parser::yes  },
+    { 'm', "map",       Arg_parser::yes },
+    { 'N', "no-trailing-nl", Arg_parser::no  },
+    { 'C', "no-version-info", Arg_parser::no  },
+    
+    {  0,  0,           Arg_parser::no  }
+};
 
 CmdLineOptions::CmdLineOptions( const int argc, const char *argv[] ):
     outputType (ansifilter::TEXT),
@@ -44,43 +83,42 @@ CmdLineOptions::CmdLineOptions( const int argc, const char *argv[] ):
     opt_linenum(false),
     opt_wrapNoNum(false),
     opt_anchors(false),
+    opt_cp437(false),
+    opt_asciiBin(false),
+    opt_asciiTundra(false),
+    opt_omit_trailing_cr(false),
+    opt_omit_version_info(false),
     encodingName("ISO-8859-1"),
     font("Courier New"),
     fontSize("10pt"),
-    wrapLineLen(0)
+    wrapLineLen(0),
+    asciiArtWidth(80),
+    asciiArtHeight(100)
 {
+    char* hlEnvOptions=getenv("ANSIFILTER_OPTIONS");
+    if (hlEnvOptions!=NULL) {
+        std::ostringstream envos;
+        envos<<argv[0]<<" "<<hlEnvOptions;
+        std::istringstream ss( envos.str());
+        std::string arg;
+        std::list<std::string> ls;
+        std::vector<char*> options;
+        while (ss >> arg)
+        {
+            ls.push_back(arg); 
+            options.push_back(const_cast<char*>(ls.back().c_str()));
+        }
+        options.push_back(0); 
+        parseRuntimeOptions(options.size()-1, (const char**) &options[0], false);
+    }
+    
+    parseRuntimeOptions(argc, argv);
+}
 
-    const Arg_parser::Option options[] = {
-        { 'a', "anchors",   Arg_parser::no },
-        { 'd', "doc-title", Arg_parser::yes },
-        { 'e', "encoding",  Arg_parser::yes },
-        { 'f', "fragment",  Arg_parser::no  },
-        { 'F', "font",      Arg_parser::yes },
-        { 'h', "help",      Arg_parser::no  },
-        { 'H', "html",      Arg_parser::no  },
-        { 'M', "pango",      Arg_parser::no  },
-        { 'i', "input",     Arg_parser::yes },
-        { 'l', "line-numbers",     Arg_parser::no },
-        { 'L', "latex",     Arg_parser::no  },
-        { 'P', "tex",       Arg_parser::no  },
-        { 'B', "bbcode",    Arg_parser::no  },
-        { 'o', "output",    Arg_parser::yes },
-        { 'O', "outdir",    Arg_parser::yes },
-        { 'p', "plain",     Arg_parser::no  },
-        { 'r', "style-ref", Arg_parser::yes },
-        { 'R', "rtf",       Arg_parser::no  },
-        { 's', "font-size", Arg_parser::yes },
-        { 't', "tail",      Arg_parser::no  },
-        { 'T', "text",      Arg_parser::no  },
-        { 'w', "wrap",      Arg_parser::yes },
-        { 'v', "version",   Arg_parser::no  },
-        { 'V', "version",   Arg_parser::no  },
-        { 'W', "wrap-no-numbers",   Arg_parser::no  },
-	{ 'm', "map",       Arg_parser::yes },
-        {  0,  0,           Arg_parser::no  }
-    };
+CmdLineOptions::~CmdLineOptions() {}
 
-    Arg_parser parser( argc, argv, options );
+void CmdLineOptions::parseRuntimeOptions( const int argc, const char *argv[], bool readInputFilenames) {
+        Arg_parser parser( argc, argv, options );
     if( parser.error().size() ) {			// bad option
         cerr << "ansifilter: "<< parser.error()<<"\n";
         cerr << "Try 'ansifilter --help' for more information.\n";
@@ -91,7 +129,7 @@ CmdLineOptions::CmdLineOptions( const int argc, const char *argv[] ):
     for( ; argind < parser.arguments(); ++argind ) {
         const int code = parser.code( argind );
         const std::string & arg = parser.argument( argind );
-        if( !code ) break;					// no more options
+        if( !code ) break;
         switch( code ) {
         /* tbd
         case 'O':
@@ -146,14 +184,12 @@ CmdLineOptions::CmdLineOptions( const int argc, const char *argv[] ):
         case 'l':
             opt_linenum=true;
             break;
-
         case 'L':
             outputType = ansifilter::LATEX;
             break;
-	    
-	case 'm':
-	    colorMapPath = arg;
-	    break;
+        case 'm':
+            colorMapPath = arg;
+            break;
         case 'M':
             outputType = ansifilter::PANGO;
             break;
@@ -194,25 +230,45 @@ CmdLineOptions::CmdLineOptions( const int argc, const char *argv[] ):
         case 'W':
             opt_wrapNoNum=true;
             break;
+        case 'X':
+            opt_cp437=true;
+            break;
+        case 'U':
+            opt_asciiBin=true;
+            break;
+        case 'D':
+            opt_asciiTundra=true;
+            break;
+        case 'Y':
+            asciiArtWidth=atoi(arg.c_str());
+            break;
+        case 'Z':
+            asciiArtHeight=atoi(arg.c_str());
+            break;
+        case 'N':
+            opt_omit_trailing_cr=true;
+            break;
+        case 'C':
+            opt_omit_version_info=true;
+            break;
         default:
             cerr << "ansifilter: option parsing failed" << endl;
         }
     }
 
-    if (argind < parser.arguments()) { //still args left
-        if  (inputFileNames.empty()) {
-            while (argind < parser.arguments()) {
-                inputFileNames.push_back( parser.argument( argind++ ) );
+    if (readInputFilenames) {
+        if (argind < parser.arguments()) { //still args left
+            if  (inputFileNames.empty()) {
+                while (argind < parser.arguments()) {
+                    inputFileNames.push_back( parser.argument( argind++ ) );
+                }
             }
+        } else if (inputFileNames.empty()) {
+            inputFileNames.push_back("");
         }
-    } else if (inputFileNames.empty()) {
-        inputFileNames.push_back("");
     }
 }
 
-CmdLineOptions::~CmdLineOptions() {}
-
-
 string CmdLineOptions::validateDirPath(const string & path)
 {
     return (path[path.length()-1] !=Platform::pathSeparator)?
@@ -271,6 +327,25 @@ bool CmdLineOptions::showLineNumbers()const
     return opt_linenum;
 }
 
+bool CmdLineOptions::parseCP437() const
+{
+  return opt_cp437;
+}
+
+bool CmdLineOptions::parseAsciiBin() const{
+  return opt_asciiBin;
+}
+
+bool CmdLineOptions::parseAsciiTundra() const{
+  return opt_asciiTundra;
+}
+int CmdLineOptions::getAsciiArtWidth() const {
+  return asciiArtWidth;   
+}
+int CmdLineOptions::getAsciiArtHeight() const{
+  return asciiArtHeight;   
+}
+
 string CmdLineOptions::getOutFileSuffix()const
 {
     switch (outputType) {
@@ -336,6 +411,14 @@ bool CmdLineOptions::omitEncoding() const
     return StringTools::lowerCase(encodingName)=="none";
 }
 
+bool CmdLineOptions::omitTrailingCR() const
+{
+    return opt_omit_trailing_cr;
+}
+bool CmdLineOptions::omitVersionInfo() const
+{
+    return opt_omit_version_info;
+}
 string CmdLineOptions::getDocumentTitle() const
 {
     return docTitle;
