@@ -2,7 +2,7 @@
                           cmdlineoptions.h  -  description
                              -------------------
     begin                : Sun Oct 13 2007
-    copyright            : (C) 2007 by Andre Simon
+    copyright            : (C) 2007-2017 by Andre Simon
     email                : andre.simon1@gmx.de
  ***************************************************************************/
 
@@ -77,7 +77,7 @@ public:
     /** \return True if help information should be printed*/
     bool printHelp() const;
 
-    /** \return True if output shluld be fragmented*/
+    /** \return True if output should be fragmented*/
     bool fragmentOutput() const;
 
     /** \return output file suffix */
@@ -96,14 +96,28 @@ public:
     bool ignoreInputEOF() const;
 
     /** \return True if line numbers should be printed */
-
     bool showLineNumbers() const;
 
+    /** \return True if anchors should be added to line numbers */
     bool addAnchors() const;
 
     /** \return True if line numbers should be printed */
     bool wrapNoNumbers() const;
 
+    /** \return True if input should be treated as codepage 437 ASCII art */
+    bool parseCP437() const;
+    
+    /** \return True if input should be treated as BIN ASCII art */
+    bool parseAsciiBin() const;
+    
+    bool parseAsciiTundra() const;
+   
+     /** \return True if output should not be terminated with carriage return */
+    bool omitTrailingCR() const;
+
+    /** \return True if output should not contain a version info comment */    
+    bool omitVersionInfo() const;
+    
     /** \return Document title */
     string getDocumentTitle() const ;
 
@@ -114,6 +128,9 @@ public:
     const vector <string> & getInputFileNames() const;
 
     int getWrapLineLength() const;
+    
+    int getAsciiArtWidth() const;
+    int getAsciiArtHeight() const;
 
 private:
     ansifilter::OutputType outputType;
@@ -126,6 +143,12 @@ private:
     bool opt_linenum;
     bool opt_wrapNoNum;
     bool opt_anchors;
+    bool opt_cp437;
+    bool opt_asciiBin;
+    bool opt_asciiTundra;
+
+    bool opt_omit_trailing_cr;
+    bool opt_omit_version_info;
 
     // name of single output file
     string outFilename;
@@ -138,6 +161,8 @@ private:
     string colorMapPath;
 
     int wrapLineLen;
+    int asciiArtWidth;
+    int asciiArtHeight;
 
     /** list of all input file names */
     vector <string> inputFileNames;
@@ -147,6 +172,9 @@ private:
 
     /** \return directory name of path */
     string getDirName( const string & path);
+    
+    void parseRuntimeOptions( const int argc, const char *argv[], bool readInputFilenames=true);
+
 };
 
 #endif
