@@ -1,7 +1,7 @@
 /***************************************************************************
                           codegenerator.h  -  description
                              -------------------
-    copyright            : (C) 2007-2016 by Andre Simon
+    copyright            : (C) 2007-2017 by Andre Simon
     email                : andre.simon1@gmx.de
  ***************************************************************************/
 
@@ -45,8 +45,13 @@ along with ANSIFilter.  If not, see <http://www.gnu.org/licenses/>.
 namespace ansifilter
 {
 
-class ReGroup;
-
+  /** TheDraw output information of individual characters*/
+  struct TDChar {
+    unsigned char c;
+    ElementStyle style;
+  };
+  
+  
 /** \brief Base class for escape sequence parsing.
 
     The virtual class provides escape sequence parsing functionality.<br>
@@ -113,7 +118,7 @@ public:
                                        const string &title);
     
     /**
-     Overrides default colours by user defined values
+     Overrides default colours by user defined values; resets palette to default if mapPath is empty
      \param mapPath path of map file
      \return true if parsing was successfull
     */
@@ -175,6 +180,18 @@ public:
         readAfterEOF=b;
     }
 
+     /** \param b set to true if the output should not be terminated with EOL*/
+    void setOmitTrailingCR(bool b)
+    {
+        omitTrailingCR=b;
+    }
+    
+    /** \param b set to true if the output should not contain a version info comment*/
+    void setOmitVersionInfo(bool b)
+    {
+        omitVersionInfo=b;
+    }
+    
     /** \return plain outputting flag */
     bool getPlainOutput()
     {
@@ -197,16 +214,34 @@ public:
     /** \param b set to true if line numbers of wrapped lines should be omitted */
     void setWrapNoNumbers(bool flag);
 
+    /** \param b set to true if input is ASCII art codepage 437 file  */
+    void setParseCodePage437(bool flag);
+
+    /** \param b set to true if input is an ASCII art BIN file*/
+    void setParseAsciiBin(bool flag);
+    
+    
+    /** \param b set to true if input is an Tundra art BIN file*/
+    void setParseAsciiTundra(bool flag);
+    
+    /** \param b set dimensions of ASCII art virtual console */
+    void setAsciiArtSize(int width, int height);
+    
 protected:
 
     /** \param type Output type */
     CodeGenerator(ansifilter::OutputType type);
-    CodeGenerator() {};
+
+    CodeGenerator() {}
 
     /** \param c Character to be masked
         \return Escape sequence of output format */
     virtual string maskCharacter(unsigned char c) = 0;
 
+    /** \param c Character to be masked
+     \return Codepage 437 escape sequence of output format */
+    virtual string maskCP437Character(unsigned char c) { return maskCharacter(c); }
+    
     /** Tag for inserting line feeds*/
     string newLineTag;
 
@@ -216,7 +251,7 @@ protected:
     /** file output*/
     ostream *out;
 
-    bool tagIsOpen;
+    bool tagIsOpen; ///< a reminder to close an open tag
 
     string styleCommentOpen,  ///< open comment delimiter
            styleCommentClose; ///< close comment delimiter
@@ -250,7 +285,12 @@ protected:
          numberWrappedLines,  ///< also show number of wrapped lines
          numberCurrentLine,   ///< output number of current line
          addAnchors;          ///< add HTML anchor to line number
-
+    
+    bool omitVersionInfo;   ///< do not print version info comment
+    bool parseCP437; ///< treat input as CP437 file
+    bool parseAsciiBin; ///< treat input as BIN or XBIN file
+    bool parseAsciiTundra; ///< treat input as Tundra file
+         
     /** Processes input data */
     void processInput();
 
@@ -262,21 +302,23 @@ protected:
         return StringTools::lowerCase(encoding)!="none";
     }
     
-        /** convert a rgb triple to HTML color notation
-        @param rgb RGB input values 
-        @return HTML color string
-        */
+    /** convert a rgb triple to HTML color notation
+    @param rgb RGB input values 
+    @return HTML color string
+    */
     string rgb2html(unsigned char* rgb);
     
+    string rgb2html(int r, int g, int b);
+    
         /// 16 basic colors
-    static unsigned char basic16[16][3];
-
+    static unsigned char workingPalette[16][3];
+    static unsigned char defaultPalette[16][3];
+    
     ElementStyle elementStyle;
 
     /** Class for line wrapping and tab replacement*/
     PreFormatter preFormatter;
-
-
+    
 private:
 
     CodeGenerator(const CodeGenerator&) {}
@@ -286,12 +328,20 @@ private:
         return *this;
     }
 
-    /** parses string to extract sequence information
+    /** parses string to extract ANSI sequence information
         @param line text line
         @param begin starting position within line
         @param end ending position within line
         @return true if sequence was recognized */
-    bool parseSequence(const string& line, size_t begin, size_t end);
+    bool parseSGRParameters(const string& line, size_t begin, size_t end);
+    
+    /** parses Codepage 437 sequence information
+        @param line text line                     
+        @param begin starting position within line
+        @param end ending position within line
+        
+        */
+    void parseCodePage437Seq(string line, size_t begin, size_t end);
 
     /** Prints document footer
         @return footer */
@@ -316,16 +366,48 @@ private:
 
     bool ignoreFormatting; ///< ignore color and font face information
     bool readAfterEOF;     ///< continue reading after EOF occoured
+    bool omitTrailingCR;   ///< do not print EOL at the end of output
 
+    
+    TDChar* termBuffer;
+    int curX, curY, memX, memY, maxY; ///< cursor position for Codepage 437 sequences
+    int asciiArtWidth;  ///< virtual console column count
+    int asciiArtHeight; ///< virtual console line count
+    
+    ElementStyle memStyle;
+    
     /** convert an xterm color value (0-253) to 3 unsigned chars rgb
         @param color xterm color
         @param rgb RGB output values */
     void xterm2rgb(unsigned char color, unsigned char* rgb);
+    
+    /**Print content of virtual terminal buffer. Deletes buffer. */
+    void printTermBuffer();
+    
+    /**Parses BIN ASCII art file */
+    void parseBinFile();
+    
+    /**Parses XBIN ASCII art file */
+    void parseXBinFile();
 
+    /**Parses Tundra ASCII art file */    
+    void parseTundraFile();
+    
+    /**allocate virtual terminal buffer */
+    void allocateTermBuffer();
+    
+    /** @return true if stream begins with XBIN id  */
+    bool streamIsXBIN();
+   
+     /** @return true if stream begins with Tundra id  */
+    bool streamIsTundra();
+   
+    
     /// the 6 value iterations in the xterm color cube
     static const unsigned char valuerange[] ;
 };
 
+
 }
 
 #endif
