@@ -1,7 +1,7 @@
 /***************************************************************************
                         mydialog.h  -  description
                              -------------------
-    copyright            : (C) 2007-2015 by Andre Simon
+    copyright            : (C) 2007-2016 by Andre Simon
     email                : andre.simon1@gmx.de
  ***************************************************************************/
 
@@ -36,13 +36,14 @@ along with ANSIFilter.  If not, see <http://www.gnu.org/licenses/>.
 #include <QtWidgets/QScrollBar>
 #endif
 
+#include "../version.h"
 #include <memory>
 #include <QClipboard>
 #include <QSettings>
 #include <QUrl>
 #include <QMimeData>
 #include <QMimeData>
-
+#include <QFileInfo>
 
 
 MyDialog::MyDialog(QWidget * parent, Qt::WindowFlags f):QDialog(parent, f)
@@ -54,13 +55,18 @@ MyDialog::MyDialog(QWidget * parent, Qt::WindowFlags f):QDialog(parent, f)
     dlg.leColorMapPath->setText(settings.value("map").toString());
     dlg.cbFragment->setChecked(settings.value("fragment").toBool());
     dlg.cbIgnoreSequences->setChecked(settings.value("ignoreseq").toBool());
+    dlg.cbParseAsciiArt->setChecked(settings.value("parseart").toBool());
+    dlg.cbOmitVersion->setChecked(settings.value("cbOmitVersion").toBool());
+    dlg.comboAnsiFormat->setCurrentIndex(settings.value("ansiformat").toInt());
     dlg.comboEncoding->setCurrentIndex(settings.value("encoding").toInt());
     dlg.comboFont->setCurrentIndex(settings.value("font").toInt());
     dlg.comboFormat->setCurrentIndex(settings.value("format").toInt());
     dlg.spinBoxWrap->setValue(settings.value("linewrap").toInt());
+    dlg.sbWidth->setValue(settings.value("width").toInt());
+    dlg.sbHeight->setValue(settings.value("height").toInt());
     settings.endGroup();
     settings.beginGroup("paths");
-    inputFileName = settings.value("infile").toString();
+    //inputFileName = settings.value("infile").toString();
     outputFileName = settings.value("outfile").toString();
     settings.endGroup();
     settings.beginGroup("window");
@@ -76,10 +82,15 @@ MyDialog::MyDialog(QWidget * parent, Qt::WindowFlags f):QDialog(parent, f)
         showFile();
         dlg.cbWatchFile->setEnabled(true);
     }
+
+//avoid ugly buttons in MacOS
+#ifndef Q_OS_OSX
+    dlg.pbSelectMapFile->setMaximumWidth(30);
+#endif
+
     plausibility();
 }
 
-
 void MyDialog::closeEvent(QCloseEvent *event)
 {
     QSettings settings("andre-simon.de", "ansifilter-gui");
@@ -88,11 +99,15 @@ void MyDialog::closeEvent(QCloseEvent *event)
     settings.setValue("map", dlg.leColorMapPath->text());
     settings.setValue("fragment", dlg.cbFragment->isChecked());
     settings.setValue("ignoreseq", dlg.cbIgnoreSequences->isChecked());
+    settings.setValue("parseart", dlg.cbParseAsciiArt->isChecked());
+    settings.setValue("cbOmitVersion", dlg.cbOmitVersion->isChecked());
+    settings.setValue("ansiformat", dlg.comboAnsiFormat->currentIndex());
     settings.setValue("encoding", dlg.comboEncoding->currentIndex());
     settings.setValue("format", dlg.comboFormat->currentIndex());
     settings.setValue("font", dlg.comboFont->currentIndex());
     settings.setValue("linewrap", dlg.spinBoxWrap->value());
-
+    settings.setValue("width", dlg.sbWidth->value());
+    settings.setValue("height", dlg.sbHeight->value());
     settings.endGroup();
     settings.beginGroup("paths");
     settings.setValue("infile", inputFileName);
@@ -121,23 +136,25 @@ void MyDialog::dragLeaveEvent(QDragLeaveEvent* event)
 
 void MyDialog::dropEvent(QDropEvent* event)
 {
-
     dlg.lblDrop->setEnabled(false);
 
     QList<QUrl> urls = event->mimeData()->urls();
     if (urls.isEmpty())
         return;
+    openFile(urls.first().toLocalFile());
+}
 
-    QString fileName = urls.first().toLocalFile();
-    if (!fileName.isEmpty()) {
-        inputFileName=fileName;
-        dlg.cbWatchFile->setEnabled(true);
-        dlg.cbWatchFile->setChecked(false);
+void MyDialog::openFile(const QString & path){
+    if (!path.isEmpty()) {
+        inputFileName=path;
+        if (!dlg.cbParseAsciiArt->isChecked()){
+            dlg.cbWatchFile->setEnabled(true);
+            dlg.cbWatchFile->setChecked(false);
+        }
         showFile();
     }
 }
 
-
 void MyDialog::onFileChanged(const QString & path)
 {
     inputFileName  = path;
@@ -149,15 +166,25 @@ void MyDialog::onFileChanged(const QString & path)
 void MyDialog::plausibility()
 {
     int selIdx = dlg.comboFormat->currentIndex();
-    dlg.cbIgnoreSequences->setEnabled(selIdx!=0);
+    dlg.cbIgnoreSequences->setEnabled(selIdx>0 && ! ((selIdx==1|| selIdx==2) && dlg.cbParseAsciiArt->isChecked() ));
     dlg.cbFragment->setEnabled(selIdx==1 || selIdx==3 || selIdx==4|| selIdx==6);
-    dlg.label->setEnabled(selIdx==1||selIdx==3);
-    dlg.comboEncoding->setEnabled(selIdx==1||selIdx==3);
+    dlg.cbParseAsciiArt->setEnabled(selIdx==1 || selIdx==2);
+
+    dlg.lblEncoding->setEnabled(selIdx==1|| selIdx==2 || selIdx==3);
+    dlg.comboEncoding->setEnabled(selIdx==1 || selIdx==2 ||selIdx==3);
     dlg.leTitle->setEnabled(selIdx==1||selIdx==3||selIdx==4);
     dlg.comboFont->setEnabled(selIdx==1||selIdx==2||selIdx==6);
+    dlg.cbOmitVersion->setEnabled(selIdx==1|| selIdx==3 || selIdx==4);
+   // dlg.gbAsciiArt->setEnabled(dlg.cbParseAsciiArt->isEnabled() && dlg.cbParseAsciiArt->isChecked());
+    dlg.comboAnsiFormat->setEnabled(dlg.cbParseAsciiArt->isEnabled() && dlg.cbParseAsciiArt->isChecked());
+    dlg.artSizeFrame->setEnabled(dlg.cbParseAsciiArt->isEnabled() && dlg.cbParseAsciiArt->isChecked());
+    dlg.lblHeight->setEnabled(dlg.gbAsciiArt->isEnabled());
+    dlg.sbHeight->setEnabled(dlg.gbAsciiArt->isEnabled());
+    dlg.lblWidth->setEnabled(dlg.gbAsciiArt->isEnabled());
+    dlg.sbWidth->setEnabled(dlg.gbAsciiArt->isEnabled());
+    dlg.cbWatchFile->setEnabled(!dlg.cbParseAsciiArt->isChecked());
 }
 
-
 ansifilter::OutputType MyDialog::getOutputType()
 {
     switch (dlg.comboFormat->currentIndex()) {
@@ -197,36 +224,57 @@ QString MyDialog::getOutFileSuffix()
     return ".txt";
 }
 
-
 void MyDialog::on_pbSaveAs_clicked()
 {
-
     if (inputFileName.isEmpty()) {
         QMessageBox::information(this, "Note", "Please select an input file.");
         return;
     }
 
+    QFileInfo check_file(dlg.leColorMapPath->text());
+    if (dlg.leColorMapPath->text().length() && (!check_file.exists() || !check_file.isFile())) {
+        QMessageBox::information(this, "Note", "Please select a color map file.");
+        dlg.leColorMapPath->setFocus();
+        return;
+    }
+
     QString outFileSuffix = getOutFileSuffix();
 
     QString outFileName =QFileDialog::getSaveFileName(this, tr("Save File"), outputFileName,
                          outFileSuffix.mid(1).toUpper() + " (*" + outFileSuffix+")" );
 
-
-    auto_ptr<ansifilter::CodeGenerator> generator(ansifilter::CodeGenerator::getInstance(getOutputType()));
+    unique_ptr<ansifilter::CodeGenerator> generator(ansifilter::CodeGenerator::getInstance(getOutputType()));
     generator->setTitle( (dlg.leTitle->text().isEmpty()? QFileInfo(outFileName).fileName() : dlg.leTitle->text()).toStdString());
     generator->setEncoding(dlg.comboEncoding->currentText().toStdString());
     generator->setFragmentCode(dlg.cbFragment->isChecked());
     generator->setPlainOutput(dlg.cbIgnoreSequences->isChecked());
+    generator->setOmitVersionInfo(dlg.cbOmitVersion->isChecked());
+
+    if (dlg.cbParseAsciiArt->isChecked()){
+        switch (dlg.comboAnsiFormat->currentIndex()){
+            case 0:
+                generator->setParseCodePage437(true);
+                break;
+            case 1:
+                generator->setParseAsciiBin(true);
+                break;
+            case 2:
+                generator->setParseAsciiTundra(true);
+                break;
+        }
+        generator->setAsciiArtSize(dlg.sbWidth->value(), dlg.sbHeight->value());
+    }
+
     generator->setFont(dlg.comboFont->currentFont().family().toStdString());
     generator->setPreformatting ( ansifilter::WRAP_SIMPLE, dlg.spinBoxWrap->value());
     generator->setFontSize("10pt"); //TODO TeX?
-    // generator->setShowLineNumbers(dlg.cbLineNumbers->isChecked());
 
-    if (!dlg.leColorMapPath->text().isEmpty()) {
-        if (!generator->setColorMap(dlg.leColorMapPath->text().toStdString()))
-            QMessageBox::warning(this, "Color Mapping Error", "Could not read color map");
+    if (!generator->setColorMap(dlg.leColorMapPath->text().toStdString())){
+        QMessageBox::warning(this, "Color Mapping Error", "Could not read color map");
         return;
     }
+
+    this->setCursor(Qt::WaitCursor);
     ansifilter::ParseError result= generator->generateFile( inputFileName.toStdString (), outFileName.toStdString () ) ;
     if (result==ansifilter::BAD_OUTPUT) {
         QMessageBox::warning(this, "IO Error", "Could not write output file");
@@ -235,6 +283,8 @@ void MyDialog::on_pbSaveAs_clicked()
     } else {
         outputFileName = outFileName;
     }
+    this->setCursor(Qt::ArrowCursor);
+
 }
 
 
@@ -242,15 +292,12 @@ void MyDialog::on_pbClipboard_clicked()
 {
 
     if (inputFileName.isEmpty()) {
-        QMessageBox::information(this, "Note",
-
-                                 "Please select an input file."
-                                );
+        QMessageBox::information(this, "Note", "Please select an input file.");
         return;
     }
-    auto_ptr<ansifilter::CodeGenerator> generator(ansifilter::CodeGenerator::getInstance(ansifilter::TEXT));
+	
+    unique_ptr<ansifilter::CodeGenerator> generator(ansifilter::CodeGenerator::getInstance(ansifilter::TEXT));
     generator->setPreformatting ( ansifilter::WRAP_SIMPLE, dlg.spinBoxWrap->value());
-    //generator->setShowLineNumbers(dlg.cbLineNumbers->isChecked());
     QString outString = QString(generator->generateStringFromFile( inputFileName.toStdString ()).c_str() ) ;
 
     if(!outString.isEmpty()) {
@@ -261,7 +308,7 @@ void MyDialog::on_pbClipboard_clicked()
 
 void MyDialog::on_pbSelectMapFile_clicked()
 {
-    QString openFile = QFileDialog::getOpenFileName(this, tr("Open Map File"), inputFileName, tr("Text files (*.*)"));
+  QString openFile = QFileDialog::getOpenFileName(this, tr("Open Map File"), "/home", tr("Text files (*.*)"));
     if (!openFile.isEmpty()) {
         dlg.leColorMapPath->setText(openFile);
         showFile();
@@ -270,13 +317,7 @@ void MyDialog::on_pbSelectMapFile_clicked()
 
 void MyDialog::on_pbFileOpen_clicked()
 {
-    QString openFile = QFileDialog::getOpenFileName(this, tr("Open File"), inputFileName, tr("Text files (*.*)"));
-    if (!openFile.isEmpty()) {
-        inputFileName = openFile;
-        dlg.cbWatchFile->setEnabled(true);
-        dlg.cbWatchFile->setChecked(false);
-        showFile();
-    }
+    openFile (QFileDialog::getOpenFileName(this, tr("Open File"), inputFileName, tr("Text files (*.*)")));
 }
 
 void MyDialog::showFile()
@@ -285,43 +326,83 @@ void MyDialog::showFile()
 
     dlg.lblInFilePath->setText(inputFileName);
 
-    auto_ptr<ansifilter::CodeGenerator> generator(ansifilter::CodeGenerator::getInstance(ansifilter::HTML));
+    unique_ptr<ansifilter::CodeGenerator> generator(ansifilter::CodeGenerator::getInstance(ansifilter::HTML));
+
     generator->setEncoding(dlg.comboEncoding->currentText().toStdString());
     generator->setFragmentCode(false);
     generator->setPlainOutput(dlg.cbIgnoreSequences->isChecked());
+
+    if (dlg.cbParseAsciiArt->isChecked()){
+        switch (dlg.comboAnsiFormat->currentIndex()){
+            case 0:
+                generator->setParseCodePage437(true);
+                break;
+            case 1:
+                generator->setParseAsciiBin(true);
+                break;
+            case 2:
+                generator->setParseAsciiTundra(true);
+                break;
+        }
+        generator->setAsciiArtSize(dlg.sbWidth->value(), dlg.sbHeight->value());
+    }
+
     generator->setFont(dlg.comboFont->currentFont().family().toStdString());
     generator->setPreformatting ( ansifilter::WRAP_SIMPLE, dlg.spinBoxWrap->value());
     generator->setFontSize("10pt");
-    if (!dlg.leColorMapPath->text().isEmpty())
-        generator->setColorMap(dlg.leColorMapPath->text().toStdString());
+    if (!generator->setColorMap(dlg.leColorMapPath->text().toStdString())){
+        QMessageBox::warning(this, "Color Mapping Error", "Could not read color map");
+        return;
+    }
 
-    //      generator->setShowLineNumbers(dlg.cbLineNumbers->isChecked());
+    this->setCursor(Qt::WaitCursor);
 
-    QString htmlString = QString( generator->generateStringFromFile(inputFileName.toStdString ()).c_str() );
+    string htmlStdString=generator->generateStringFromFile(inputFileName.toStdString ());
+    QString htmlString( htmlStdString.c_str() );
     if (!htmlString.isEmpty()) {
         dlg.textEdit->setText(htmlString);
         this->setWindowTitle("ANSIFilter - " + inputFileName);
     }
+    this->setCursor(Qt::ArrowCursor);
 }
 
 void MyDialog::on_pbAbout_clicked()
 {
     QMessageBox::about(this,
-
-                       "ANSIFilter Information", "ANSIFilter GUI Version 1.16\n"
-                       "(c) 2007-2016 Andre Simon\n\n"
+                       "ANSIFilter Information", 
+                       QString("ANSIFilter GUI %1\n" 
+                       "(c) 2007-2018 Andre Simon\n\n"
+                       "Built with Qt version %2\n\n"
                        "Released under the terms of the GNU GPL license.\n\n"
                        "andre dot simon1 at gmx dot de\n"
-                       "See www.andre-simon.de for updates."
+                       "See www.andre-simon.de for updates.").arg(ANSIFILTER_VERSION).arg(QString(qVersion ())) 
                       );
 }
 
 void MyDialog::on_cbIgnoreSequences_stateChanged()
 {
-
     showFile();
 }
 
+void MyDialog::on_cbParseAsciiArt_clicked()
+{
+    plausibility();
+    showFile();
+}
+
+
+void MyDialog::on_sbWidth_valueChanged(int i)
+{
+    showFile();
+}
+void MyDialog::on_sbHeight_valueChanged(int i)
+{
+    showFile();
+}
+void MyDialog::on_comboAnsiFormat_currentIndexChanged(int idx)
+{
+    showFile();
+}
 void MyDialog::on_comboFont_currentIndexChanged(int idx)
 {
     showFile();
